@@ -13,6 +13,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 namespace base {
 
+using std::begin;
+using std::end;
+
 template <
 	typename Key,
 	typename Type,
@@ -414,8 +417,9 @@ public:
 		if (range.first == range.second) {
 			return 0;
 		}
+		const auto result = (range.second - range.first);
 		impl().erase(range.first, range.second);
-		return (range.second - range.first);
+		return result;
 	}
 
 	iterator erase(const_iterator where) {
@@ -424,6 +428,9 @@ public:
 	iterator erase(const_iterator from, const_iterator till) {
 		return impl().erase(from._impl, till._impl);
 	}
+	int erase(const Key &key) {
+		return removeAll(key);
+	}
 
 	iterator findFirst(const Key &key) {
 		if (empty()
