@@ -109,6 +109,15 @@ public:
 		return _impl.template get_unchecked<T>();
 	}
 
+	template <typename ...Methods>
+	decltype(auto) match(Methods &&...methods) {
+		return base::match(_impl, std::forward<Methods>(methods)...);
+	}
+	template <typename ...Methods>
+	decltype(auto) match(Methods &&...methods) const {
+		return base::match(_impl, std::forward<Methods>(methods)...);
+	}
+
 private:
 	variant<none_type, Types...> _impl;
 
@@ -124,6 +133,20 @@ inline const T *get_if(const optional_variant<Types...> *v) {
 	return (v && v->template is<T>()) ? &v->template get_unchecked<T>() : nullptr;
 }
 
+template <typename ...Types, typename ...Methods>
+inline decltype(auto) match(
+		optional_variant<Types...> &value,
+		Methods &&...methods) {
+	return value.match(std::forward<Methods>(methods)...);
+}
+
+template <typename ...Types, typename ...Methods>
+inline decltype(auto) match(
+		const optional_variant<Types...> &value,
+		Methods &&...methods) {
+	return value.match(std::forward<Methods>(methods)...);
+}
+
 template <typename Type>
 class optional;
 
@@ -160,12 +183,17 @@ class optional : public optional_variant<Type> {
 public:
 	using parent::parent;
 
-	Type &operator*() {
+	Type &operator*() & {
 		Expects(parent::template is<Type>());
 
 		return parent::template get_unchecked<Type>();
 	}
-	const Type &operator*() const {
+	Type &&operator*() && {
+		Expects(parent::template is<Type>());
+
+		return std::move(parent::template get_unchecked<Type>());
+	}
+	const Type &operator*() const & {
 		Expects(parent::template is<Type>());
 
 		return parent::template get_unchecked<Type>();
