@@ -66,7 +66,6 @@ class EditParticipantBox::Inner : public TWidget {
 public:
 	Inner(
 		QWidget *parent,
-		not_null<Window::Controller*> controller,
 		not_null<ChannelData*> channel,
 		not_null<UserData*> user,
 		bool hasAdminRights);
@@ -101,7 +100,6 @@ private:
 
 EditParticipantBox::Inner::Inner(
 	QWidget *parent,
-	not_null<Window::Controller*> controller,
 	not_null<ChannelData*> channel,
 	not_null<UserData*> user,
 	bool hasAdminRights)
@@ -110,7 +108,6 @@ EditParticipantBox::Inner::Inner(
 , _user(user)
 , _userPhoto(
 	this,
-	controller,
 	_user,
 	Ui::UserpicButton::Role::Custom,
 	st::rightsPhotoButton)
@@ -184,7 +181,6 @@ EditParticipantBox::EditParticipantBox(QWidget*, not_null<ChannelData*> channel,
 void EditParticipantBox::prepare() {
 	_inner = setInnerWidget(object_ptr<Inner>(
 		this,
-		controller(),
 		_channel,
 		_user,
 		hasAdminRights()));
