@@ -10,23 +10,16 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_types.h"
 
 class AuthSession;
+class mtpFileLoader;
 
 inline uint64 mediaMix32To64(int32 a, int32 b) {
 	return (uint64(*reinterpret_cast<uint32*>(&a)) << 32)
 		| uint64(*reinterpret_cast<uint32*>(&b));
 }
 
-// Old method, should not be used anymore.
-//inline MediaKey mediaKey(LocationType type, int32 dc, const uint64 &id) {
-//	return MediaKey(mediaMix32To64(type, dc), id);
-//}
-// New method when version was introduced, type is not relevant anymore (all files are Documents).
-inline MediaKey mediaKey(
-		LocationType type,
-		int32 dc,
-		const uint64 &id,
-		int32 version) {
-	return (version > 0) ? MediaKey(mediaMix32To64(version, dc), id) : MediaKey(mediaMix32To64(type, dc), id);
+// version field removed from document.
+inline MediaKey mediaKey(LocationType type, int32 dc, const uint64 &id) {
+	return MediaKey(mediaMix32To64(type, dc), id);
 }
 
 inline StorageKey mediaKey(const MTPDfileLocation &location) {
@@ -42,11 +35,12 @@ struct DocumentAdditionalData {
 };
 
 struct StickerData : public DocumentAdditionalData {
+	Data::FileOrigin setOrigin() const;
+
 	ImagePtr img;
 	QString alt;
 	MTPInputStickerSet set = MTP_inputStickerSetEmpty();
 	StorageImageLocation loc; // doc thumb location
-
 };
 
 struct SongData : public DocumentAdditionalData {
@@ -78,7 +72,9 @@ public:
 	void setattributes(
 		const QVector<MTPDocumentAttribute> &attributes);
 
-	void automaticLoad(const HistoryItem *item); // auto load sticker or video
+	void automaticLoad(
+		Data::FileOrigin origin,
+		const HistoryItem *item); // auto load sticker or video
 	void automaticLoadSettingsChanged();
 
 	enum FilePathResolveType {
@@ -93,6 +89,7 @@ public:
 	QString loadingFilePath() const;
 	bool displayLoading() const;
 	void save(
+		Data::FileOrigin origin,
 		const QString &toFile,
 		ActionOnLoad action = ActionOnLoadNone,
 		const FullMsgId &actionMsgId = FullMsgId(),
@@ -119,10 +116,14 @@ public:
 	void performActionOnLoad();
 
 	void forget();
-	ImagePtr makeReplyPreview();
+	ImagePtr makeReplyPreview(Data::FileOrigin origin);
 
 	StickerData *sticker() const;
 	void checkSticker();
+	void checkStickerThumb();
+	ImagePtr getStickerThumb();
+	Data::FileOrigin stickerSetOrigin() const;
+	Data::FileOrigin stickerOrGifOrigin() const;
 	bool isStickerSetInstalled() const;
 	SongData *song();
 	const SongData *song() const;
@@ -147,14 +148,18 @@ public:
 
 	bool hasGoodStickerThumb() const;
 
-	bool setRemoteVersion(int32 version); // Returns true if version has changed.
-	void setRemoteLocation(int32 dc, uint64 access);
+	void setRemoteLocation(
+		int32 dc,
+		uint64 access,
+		const QByteArray &fileReference);
 	void setContentUrl(const QString &url);
 	void setWebLocation(const WebFileLocation &location);
 	bool hasRemoteLocation() const;
 	bool hasWebLocation() const;
 	bool isValid() const;
 	MTPInputDocument mtpInput() const;
+	QByteArray fileReference() const;
+	void refreshFileReference(const QByteArray &value);
 
 	// When we have some client-side generated document
 	// (for example for displaying an external inline bot result)
@@ -195,10 +200,10 @@ private:
 
 	void destroyLoaderDelayed(mtpFileLoader *newValue = nullptr) const;
 
-	// Two types of location: from MTProto by dc+access+version or from web by url
+	// Two types of location: from MTProto by dc+access or from web by url
 	int32 _dc = 0;
 	uint64 _access = 0;
-	int32 _version = 0;
+	QByteArray _fileReference;
 	QString _url;
 	QString _filename;
 	QString _mimeString;
@@ -240,7 +245,8 @@ private:
 class DocumentSaveClickHandler : public DocumentClickHandler {
 public:
 	using DocumentClickHandler::DocumentClickHandler;
-	static void doSave(
+	static void Save(
+		Data::FileOrigin origin,
 		not_null<DocumentData*> document,
 		bool forceSavingAs = false);
 
@@ -252,7 +258,8 @@ protected:
 class DocumentOpenClickHandler : public DocumentClickHandler {
 public:
 	using DocumentClickHandler::DocumentClickHandler;
-	static void doOpen(
+	static void Open(
+		Data::FileOrigin origin,
 		not_null<DocumentData*> document,
 		HistoryItem *context,
 		ActionOnLoad action = ActionOnLoadOpen);
