@@ -150,7 +150,7 @@ void Feed::unregisterOne(not_null<ChannelData*> channel) {
 void Feed::updateLastMessage(not_null<HistoryItem*> item) {
 	if (justUpdateLastMessage(item)) {
 		if (_lastMessage && *_lastMessage) {
-			setChatsListDate(ItemDateTime(*_lastMessage));
+			setChatsListTimeId((*_lastMessage)->date());
 		}
 	}
 }
@@ -304,7 +304,7 @@ void Feed::setLastMessageFromChannels() {
 
 void Feed::updateChatsListDate() {
 	if (_lastMessage && *_lastMessage) {
-		setChatsListDate(ItemDateTime(*_lastMessage));
+		setChatsListTimeId((*_lastMessage)->date());
 	}
 }
 
