@@ -57,7 +57,7 @@ void UpdateImage(ImagePtr &old, ImagePtr now) {
 	} else if (const auto delayed = old->toDelayedStorageImage()) {
 		const auto location = now->location();
 		if (!location.isNull()) {
-			delayed->setStorageLocation(location);
+			delayed->setStorageLocation(Data::FileOrigin(), location);
 		}
 	}
 }
@@ -72,12 +72,16 @@ Session::Session(not_null<AuthSession*> session)
 	setupChannelLeavingViewer();
 }
 
-void Session::startExport() {
+void Session::startExport(PeerData *peer) {
+	startExport(peer ? peer->input : MTP_inputPeerEmpty());
+}
+
+void Session::startExport(const MTPInputPeer &singlePeer) {
 	if (_exportPanel) {
 		_exportPanel->activatePanel();
 		return;
 	}
-	_export = std::make_unique<Export::ControllerWrap>();
+	_export = std::make_unique<Export::ControllerWrap>(singlePeer);
 	_exportPanel = std::make_unique<Export::View::PanelController>(
 		_export.get());
 
@@ -700,7 +704,7 @@ void Session::updateNotifySettingsLocal(not_null<PeerData*> peer) {
 }
 
 void Session::unmuteByFinishedDelayed(TimeMs delay) {
-	accumulate_max(delay, kMaxNotifyCheckDelay);
+	accumulate_min(delay, kMaxNotifyCheckDelay);
 	if (!_unmuteByFinishedTimer.isActive()
 		|| _unmuteByFinishedTimer.remainingTime() > delay) {
 		_unmuteByFinishedTimer.callOnce(delay);
@@ -795,6 +799,7 @@ not_null<PhotoData*> Session::photo(
 		return photo(
 			data.c_photo().vid.v,
 			data.c_photo().vaccess_hash.v,
+			data.c_photo().vfile_reference.v,
 			data.c_photo().vdate.v,
 			ImagePtr(*thumb, "JPG"),
 			ImagePtr(*medium, "JPG"),
@@ -809,6 +814,7 @@ not_null<PhotoData*> Session::photo(
 not_null<PhotoData*> Session::photo(
 		PhotoId id,
 		const uint64 &access,
+		const QByteArray &fileReference,
 		TimeId date,
 		const ImagePtr &thumb,
 		const ImagePtr &medium,
@@ -817,6 +823,7 @@ not_null<PhotoData*> Session::photo(
 	photoApplyFields(
 		result,
 		access,
+		fileReference,
 		date,
 		thumb,
 		medium,
@@ -874,6 +881,7 @@ PhotoData *Session::photoFromWeb(
 	return photo(
 		rand_value<PhotoId>(),
 		uint64(0),
+		QByteArray(),
 		unixtime(),
 		thumb,
 		medium,
@@ -937,6 +945,7 @@ void Session::photoApplyFields(
 		photoApplyFields(
 			photo,
 			data.vaccess_hash.v,
+			data.vfile_reference.v,
 			data.vdate.v,
 			App::image(*thumb),
 			App::image(*medium),
@@ -947,6 +956,7 @@ void Session::photoApplyFields(
 void Session::photoApplyFields(
 		not_null<PhotoData*> photo,
 		const uint64 &access,
+		const QByteArray &fileReference,
 		TimeId date,
 		const ImagePtr &thumb,
 		const ImagePtr &medium,
@@ -955,6 +965,7 @@ void Session::photoApplyFields(
 		return;
 	}
 	photo->access = access;
+	photo->fileReference = fileReference;
 	photo->date = date;
 	UpdateImage(photo->thumb, thumb);
 	UpdateImage(photo->medium, medium);
@@ -1000,7 +1011,7 @@ not_null<DocumentData*> Session::document(
 		return document(
 			fields.vid.v,
 			fields.vaccess_hash.v,
-			fields.vversion.v,
+			fields.vfile_reference.v,
 			fields.vdate.v,
 			fields.vattributes.v,
 			qs(fields.vmime_type),
@@ -1016,7 +1027,7 @@ not_null<DocumentData*> Session::document(
 not_null<DocumentData*> Session::document(
 		DocumentId id,
 		const uint64 &access,
-		int32 version,
+		const QByteArray &fileReference,
 		TimeId date,
 		const QVector<MTPDocumentAttribute> &attributes,
 		const QString &mime,
@@ -1028,7 +1039,7 @@ not_null<DocumentData*> Session::document(
 	documentApplyFields(
 		result,
 		access,
-		version,
+		fileReference,
 		date,
 		attributes,
 		mime,
@@ -1106,7 +1117,7 @@ DocumentData *Session::documentFromWeb(
 	const auto result = document(
 		rand_value<DocumentId>(),
 		uint64(0),
-		int32(0),
+		QByteArray(),
 		unixtime(),
 		data.vattributes.v,
 		data.vmime_type.v,
@@ -1127,7 +1138,7 @@ DocumentData *Session::documentFromWeb(
 	const auto result = document(
 		rand_value<DocumentId>(),
 		uint64(0),
-		int32(0),
+		QByteArray(),
 		unixtime(),
 		data.vattributes.v,
 		data.vmime_type.v,
@@ -1153,7 +1164,7 @@ void Session::documentApplyFields(
 	documentApplyFields(
 		document,
 		data.vaccess_hash.v,
-		data.vversion.v,
+		data.vfile_reference.v,
 		data.vdate.v,
 		data.vattributes.v,
 		qs(data.vmime_type),
@@ -1166,7 +1177,7 @@ void Session::documentApplyFields(
 void Session::documentApplyFields(
 		not_null<DocumentData*> document,
 		const uint64 &access,
-		int32 version,
+		const QByteArray &fileReference,
 		TimeId date,
 		const QVector<MTPDocumentAttribute> &attributes,
 		const QString &mime,
@@ -1178,9 +1189,8 @@ void Session::documentApplyFields(
 		return;
 	}
 	document->setattributes(attributes);
-	document->setRemoteVersion(version);
 	if (dc != 0 && access != 0) {
-		document->setRemoteLocation(dc, access);
+		document->setRemoteLocation(dc, access, fileReference);
 	}
 	document->date = date;
 	document->setMimeString(mime);
@@ -1349,9 +1359,7 @@ void Session::webpageApplyFields(
 		int duration,
 		const QString &author,
 		TimeId pendingTill) {
-	if (!page->pendingTill && pendingTill > 0) {
-		_session->api().requestWebPageDelayed(page);
-	}
+	const auto requestPending = (!page->pendingTill && pendingTill > 0);
 	const auto changed = page->applyChanges(
 		type,
 		url,
@@ -1364,6 +1372,9 @@ void Session::webpageApplyFields(
 		duration,
 		author,
 		pendingTill);
+	if (requestPending) {
+		_session->api().requestWebPageDelayed(page);
+	}
 	if (changed) {
 		notifyWebPageUpdateDelayed(page);
 	}
