@@ -34,7 +34,7 @@ public:
 	SectionToggle(
 		const style::InfoToggle &st,
 		bool checked,
-		base::lambda<void()> updateCallback);
+		Fn<void()> updateCallback);
 
 	QSize getSize() const override;
 	void paint(
@@ -56,7 +56,7 @@ private:
 SectionToggle::SectionToggle(
 		const style::InfoToggle &st,
 		bool checked,
-		base::lambda<void()> updateCallback)
+		Fn<void()> updateCallback)
 : AbstractCheckView(st.duration, checked, std::move(updateCallback))
 , _st(st) {
 }
@@ -248,21 +248,11 @@ Cover::Cover(
 
 void Cover::setupChildGeometry() {
 	using namespace rpl::mappers;
-	//
-	// Visual Studio 2017 15.5.1 internal compiler error here.
-	// See https://developercommunity.visualstudio.com/content/problem/165155/ice-regression-in-1551-after-successfull-build-in.html
-	//
-	//rpl::combine(
-	//	toggleShownValue(),
-	//	widthValue(),
-	//	_2
-	//) | rpl::map([](bool shown, int width) {
 	rpl::combine(
 		toggleShownValue(),
-		widthValue()
-	) | rpl::map([](bool shown, int width) {
-		return width;
-	}) | rpl::start_with_next([this](int newWidth) {
+		widthValue(),
+		_2
+	) | rpl::start_with_next([this](int newWidth) {
 		_userpic->moveToLeft(
 			st::infoProfilePhotoLeft,
 			st::infoProfilePhotoTop,
@@ -453,21 +443,12 @@ void SharedMediaCover::createLabel() {
 		Lang::Viewer(lng_profile_shared_media) | ToUpperValue(),
 		st::infoBlockHeaderLabel);
 	label->setAttribute(Qt::WA_TransparentForMouseEvents);
-	//
-	// Visual Studio 2017 15.5.1 internal compiler error here.
-	// See https://developercommunity.visualstudio.com/content/problem/165155/ice-regression-in-1551-after-successfull-build-in.html
-	//
-	//rpl::combine(
-	//	toggleShownValue(),
-	//	widthValue(),
-	//	_2
-	//) | rpl::map([](bool shown, int width) {
+
 	rpl::combine(
 		toggleShownValue(),
-		widthValue()
-	) | rpl::map([](bool shown, int width) {
-		return width;
-	}) | rpl::start_with_next([this, weak = label.data()](int newWidth) {
+		widthValue(),
+		_2
+	) | rpl::start_with_next([this, weak = label.data()](int newWidth) {
 		auto availableWidth = newWidth
 			- st::infoBlockHeaderPosition.x()
 			- st::infoSharedMediaButton.padding.right()
