@@ -22,6 +22,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/view/history_view_cursor_state.h"
 #include "storage/localstorage.h"
 #include "auth_session.h"
+#include "apiwrap.h"
 #include "lang/lang_keys.h"
 
 namespace InlineBots {
@@ -117,12 +118,12 @@ void Gif::setPosition(int32 position) {
 }
 
 void DeleteSavedGifClickHandler::onClickImpl() const {
-	auto index = Auth().data().savedGifs().indexOf(_data);
+	Auth().api().toggleSavedGif(_data, Data::FileOriginSavedGifs(), false);
+
+	const auto index = Auth().data().savedGifs().indexOf(_data);
 	if (index >= 0) {
 		Auth().data().savedGifsRef().remove(index);
 		Local::writeSavedGifs();
-
-		MTP::send(MTPmessages_SaveGif(_data->mtpInput(), MTP_bool(true)));
 	}
 	Auth().data().notifySavedGifsUpdated();
 }
@@ -134,8 +135,8 @@ int Gif::resizeGetHeight(int width) {
 }
 
 void Gif::paint(Painter &p, const QRect &clip, const PaintContext *context) const {
-	DocumentData *document = getShownDocument();
-	document->automaticLoad(nullptr);
+	const auto document = getShownDocument();
+	document->automaticLoad(fileOrigin(), nullptr);
 
 	bool loaded = document->loaded(), loading = document->loading(), displayLoading = document->displayLoading();
 	if (loaded && !_gif && !_gif.isBad()) {
@@ -286,25 +287,26 @@ QSize Gif::countFrameSize() const {
 }
 
 void Gif::prepareThumb(int32 width, int32 height, const QSize &frame) const {
-	if (DocumentData *document = getShownDocument()) {
+	const auto origin = fileOrigin();
+	if (const auto document = getShownDocument()) {
 		if (!document->thumb->isNull()) {
 			if (document->thumb->loaded()) {
 				if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
-					_thumb = document->thumb->pixNoCache(frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+					_thumb = document->thumb->pixNoCache(origin, frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 				}
 			} else {
-				document->thumb->load();
+				document->thumb->load(origin);
 			}
 		}
 	} else {
-		ImagePtr thumb = getResultThumb();
+		const auto thumb = getResultThumb();
 		if (!thumb->isNull()) {
 			if (thumb->loaded()) {
 				if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
-					_thumb = thumb->pixNoCache(frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+					_thumb = thumb->pixNoCache(origin, frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 				}
 			} else {
-				thumb->load();
+				thumb->load(origin);
 			}
 		}
 	}
@@ -375,17 +377,9 @@ void Sticker::initDimensions() {
 
 void Sticker::preload() const {
 	if (const auto document = getShownDocument()) {
-		const auto goodThumb = document->hasGoodStickerThumb();
-		if (goodThumb) {
-			document->thumb->load();
-		} else {
-			document->checkSticker();
-		}
-	} else {
-		const auto thumb = getResultThumb();
-		if (!thumb->isNull()) {
-			thumb->load();
-		}
+		document->checkStickerThumb();
+	} else if (const auto thumb = getResultThumb()) {
+		thumb->load(fileOrigin());
 	}
 }
 
@@ -441,33 +435,31 @@ QSize Sticker::getThumbSize() const {
 
 void Sticker::prepareThumb() const {
 	if (const auto document = getShownDocument()) {
-		const auto goodThumb = document->hasGoodStickerThumb();
-		if (goodThumb) {
-			document->thumb->load();
-		} else {
-			document->checkSticker();
-		}
+		document->checkStickerThumb();
 
-		const auto sticker = goodThumb
-			? document->thumb
-			: document->sticker()
-			? document->sticker()->img
-			: ImagePtr();
+		const auto sticker = document->getStickerThumb();
 		if (!_thumbLoaded && !sticker->isNull() && sticker->loaded()) {
-			QSize thumbSize = getThumbSize();
-			_thumb = sticker->pix(thumbSize.width(), thumbSize.height());
+			const auto thumbSize = getThumbSize();
+			_thumb = sticker->pix(
+				document->stickerSetOrigin(),
+				thumbSize.width(),
+				thumbSize.height());
 			_thumbLoaded = true;
 		}
 	} else {
-		ImagePtr thumb = getResultThumb();
+		const auto origin = fileOrigin();
+		const auto thumb = getResultThumb();
 		if (thumb->loaded()) {
 			if (!_thumbLoaded) {
-				QSize thumbSize = getThumbSize();
-				_thumb = thumb->pix(thumbSize.width(), thumbSize.height());
+				const auto thumbSize = getThumbSize();
+				_thumb = thumb->pix(
+					origin,
+					thumbSize.width(),
+					thumbSize.height());
 				_thumbLoaded = true;
 			}
 		} else {
-			thumb->load();
+			thumb->load(origin);
 		}
 	}
 }
@@ -546,28 +538,29 @@ QSize Photo::countFrameSize() const {
 }
 
 void Photo::prepareThumb(int32 width, int32 height, const QSize &frame) const {
-	if (PhotoData *photo = getShownPhoto()) {
+	const auto origin = fileOrigin();
+	if (const auto photo = getShownPhoto()) {
 		if (photo->medium->loaded()) {
 			if (!_thumbLoaded || _thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
-				_thumb = photo->medium->pixNoCache(frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+				_thumb = photo->medium->pixNoCache(origin, frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 			}
 			_thumbLoaded = true;
 		} else {
 			if (photo->thumb->loaded()) {
 				if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
-					_thumb = photo->thumb->pixNoCache(frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+					_thumb = photo->thumb->pixNoCache(origin, frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 				}
 			}
-			photo->medium->load();
+			photo->medium->load(origin);
 		}
 	} else {
-		ImagePtr thumb = getResultThumb();
+		const auto thumb = getResultThumb();
 		if (thumb->loaded()) {
 			if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
-				_thumb = thumb->pixNoCache(frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+				_thumb = thumb->pixNoCache(origin, frame.width() * cIntRetinaFactor(), frame.height() * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 			}
 		} else {
-			thumb->load();
+			thumb->load(origin);
 		}
 	}
 }
@@ -660,7 +653,8 @@ TextState Video::getState(
 }
 
 void Video::prepareThumb(int32 width, int32 height) const {
-	ImagePtr thumb = content_thumb();
+	const auto thumb = content_thumb();
+	const auto origin = fileOrigin();
 	if (thumb->loaded()) {
 		if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
 			int32 w = qMax(convertScale(thumb->width()), 1), h = qMax(convertScale(thumb->height()), 1);
@@ -675,10 +669,10 @@ void Video::prepareThumb(int32 width, int32 height) const {
 					w = width;
 				}
 			}
-			_thumb = thumb->pixNoCache(w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+			_thumb = thumb->pixNoCache(origin, w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 		}
 	} else {
-		thumb->load();
+		thumb->load(origin);
 	}
 }
 
@@ -971,7 +965,7 @@ TextState Contact::getState(
 }
 
 void Contact::prepareThumb(int width, int height) const {
-	ImagePtr thumb = getResultThumb();
+	const auto thumb = getResultThumb();
 	if (thumb->isNull()) {
 		if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
 			_thumb = getResultContactAvatar(width, height);
@@ -979,6 +973,7 @@ void Contact::prepareThumb(int width, int height) const {
 		return;
 	}
 
+	const auto origin = fileOrigin();
 	if (thumb->loaded()) {
 		if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
 			int w = qMax(convertScale(thumb->width()), 1), h = qMax(convertScale(thumb->height()), 1);
@@ -993,10 +988,10 @@ void Contact::prepareThumb(int width, int height) const {
 					w = width;
 				}
 			}
-			_thumb = thumb->pixNoCache(w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+			_thumb = thumb->pixNoCache(origin, w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 		}
 	} else {
-		thumb->load();
+		thumb->load(origin);
 	}
 }
 
@@ -1118,7 +1113,7 @@ TextState Article::getState(
 }
 
 void Article::prepareThumb(int width, int height) const {
-	ImagePtr thumb = getResultThumb();
+	const auto thumb = getResultThumb();
 	if (thumb->isNull()) {
 		if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
 			_thumb = getResultContactAvatar(width, height);
@@ -1126,6 +1121,7 @@ void Article::prepareThumb(int width, int height) const {
 		return;
 	}
 
+	const auto origin = fileOrigin();
 	if (thumb->loaded()) {
 		if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
 			int w = qMax(convertScale(thumb->width()), 1), h = qMax(convertScale(thumb->height()), 1);
@@ -1140,10 +1136,10 @@ void Article::prepareThumb(int width, int height) const {
 					w = width;
 				}
 			}
-			_thumb = thumb->pixNoCache(w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+			_thumb = thumb->pixNoCache(origin, w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 		}
 	} else {
-		thumb->load();
+		thumb->load(origin);
 	}
 }
 
@@ -1217,7 +1213,7 @@ void Game::paint(Painter &p, const QRect &clip, const PaintContext *context) con
 	auto document = getResultDocument();
 	auto animatedThumb = document && document->isAnimation();
 	if (animatedThumb) {
-		document->automaticLoad(nullptr);
+		document->automaticLoad(fileOrigin(), nullptr);
 
 		bool loaded = document->loaded(), loading = document->loading(), displayLoading = document->displayLoading();
 		if (loaded && !_gif && !_gif.isBad()) {
@@ -1293,7 +1289,7 @@ TextState Game::getState(
 }
 
 void Game::prepareThumb(int width, int height) const {
-	auto thumb = [this] {
+	const auto thumb = [this] {
 		if (auto photo = getResultPhoto()) {
 			return photo->medium;
 		} else if (auto document = getResultDocument()) {
@@ -1305,6 +1301,7 @@ void Game::prepareThumb(int width, int height) const {
 		return;
 	}
 
+	const auto origin = fileOrigin();
 	if (thumb->loaded()) {
 		if (_thumb.width() != width * cIntRetinaFactor() || _thumb.height() != height * cIntRetinaFactor()) {
 			int w = qMax(convertScale(thumb->width()), 1), h = qMax(convertScale(thumb->height()), 1);
@@ -1321,10 +1318,10 @@ void Game::prepareThumb(int width, int height) const {
 					w = width;
 				}
 			}
-			_thumb = thumb->pixNoCache(w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
+			_thumb = thumb->pixNoCache(origin, w * cIntRetinaFactor(), h * cIntRetinaFactor(), Images::Option::Smooth, width, height);
 		}
 	} else {
-		thumb->load();
+		thumb->load(origin);
 	}
 }
 
