@@ -112,8 +112,12 @@ public:
 		anim::type animated);
 	void ui_hideSettingsAndLayer(anim::type animated);
 	bool ui_isLayerShown();
-	void ui_showMediaPreview(DocumentData *document);
-	void ui_showMediaPreview(PhotoData *photo);
+	void ui_showMediaPreview(
+		Data::FileOrigin origin,
+		not_null<DocumentData*> document);
+	void ui_showMediaPreview(
+		Data::FileOrigin origin,
+		not_null<PhotoData*> photo);
 	void ui_hideMediaPreview();
 
 protected:
