@@ -78,6 +78,9 @@ MediaView::MediaView()
 , _dropdownShowTimer(this) {
 	subscribe(Lang::Current().updated(), [this] { refreshLang(); });
 
+	setWindowIcon(Window::CreateIcon());
+	setWindowTitle(qsl("Media viewer"));
+
 	TextCustomTagsMap custom;
 	custom.insert(QChar('c'), qMakePair(textcmdStartLink(1), textcmdStopLink()));
 	_saveMsgText.setRichText(st::mediaviewSaveMsgStyle, lang(lng_mediaview_saved), Ui::DialogTextOptions(), custom);
@@ -117,7 +120,7 @@ MediaView::MediaView()
 	});
 	handleAuthSessionChange();
 
-	setWindowFlags(Qt::FramelessWindowHint | Qt::BypassWindowManagerHint | Qt::Tool | Qt::NoDropShadowWindowHint);
+	setWindowFlags(Qt::FramelessWindowHint);
 	moveToScreen();
 	setAttribute(Qt::WA_NoSystemBackground, true);
 	setAttribute(Qt::WA_TranslucentBackground, true);
@@ -125,7 +128,11 @@ MediaView::MediaView()
 
 	hide();
 	createWinId();
-	if (cPlatform() == dbipWindows) {
+	if (cPlatform() == dbipLinux32 || cPlatform() == dbipLinux64) {
+		windowHandle()->setTransientParent(App::wnd()->windowHandle());
+		setWindowModality(Qt::WindowModal);
+	}
+	if (cPlatform() != dbipMac && cPlatform() != dbipMacOld) {
 		setWindowState(Qt::WindowFullScreen);
 	}
 
@@ -165,7 +172,8 @@ void MediaView::moveToScreen() {
 	if (activeWindowScreen && myScreen && myScreen != activeWindowScreen) {
 		windowHandle()->setScreen(activeWindowScreen);
 	}
-	auto available = activeWindow ? Sandbox::screenGeometry(activeWindow->geometry().center()) : QApplication::desktop()->screenGeometry();
+	const auto screen = activeWindowScreen ? activeWindowScreen : QApplication::primaryScreen();
+	const auto available = screen->geometry();
 	if (geometry() != available) {
 		setGeometry(available);
 	}
@@ -838,7 +846,7 @@ void MediaView::onSaveAs() {
 			if (_doc->data().isEmpty()) location.accessDisable();
 		} else {
 			if (!fileShown()) {
-				DocumentSaveClickHandler::doSave(_doc, true);
+				DocumentSaveClickHandler::Save(fileOrigin(), _doc, true);
 				updateControls();
 			} else {
 				_saveVisible = false;
@@ -863,7 +871,7 @@ void MediaView::onSaveAs() {
 				_photo->date),
 			crl::guard(this, [this, photo = _photo](const QString &result) {
 				if (!result.isEmpty() && _photo == photo && photo->loaded()) {
-					photo->full->pix().toImage().save(result, "JPG");
+					photo->full->pix(fileOrigin()).toImage().save(result, "JPG");
 				}
 				psShowOverAll(this);
 			}), crl::guard(this, [this] {
@@ -879,7 +887,11 @@ void MediaView::onDocClick() {
 	if (_doc->loading()) {
 		onSaveCancel();
 	} else {
-		DocumentOpenClickHandler::doOpen(_doc, nullptr, ActionOnLoadNone);
+		DocumentOpenClickHandler::Open(
+			fileOrigin(),
+			_doc,
+			App::histItemById(_msgid),
+			ActionOnLoadNone);
 		if (_doc->loading() && !_radial.animating()) {
 			_radial.start(_doc->progress());
 		}
@@ -966,7 +978,7 @@ void MediaView::onDownload() {
 			location.accessDisable();
 		} else {
 			if (!fileShown()) {
-				DocumentSaveClickHandler::doSave(_doc);
+				DocumentSaveClickHandler::Save(fileOrigin(), _doc);
 				updateControls();
 			} else {
 				_saveVisible = false;
@@ -981,7 +993,7 @@ void MediaView::onDownload() {
 		} else {
 			if (!QDir().exists(path)) QDir().mkpath(path);
 			toName = filedialogDefaultName(qsl("photo"), qsl(".jpg"), path);
-			if (!_photo->full->pix().toImage().save(toName, "JPG")) {
+			if (!_photo->full->pix(fileOrigin()).toImage().save(toName, "JPG")) {
 				toName = QString();
 			}
 		}
@@ -1060,7 +1072,7 @@ void MediaView::onCopy() {
 	} else {
 		if (!_photo || !_photo->loaded()) return;
 
-		QApplication::clipboard()->setPixmap(_photo->full->pix());
+		QApplication::clipboard()->setPixmap(_photo->full->pix(fileOrigin()));
 	}
 }
 
@@ -1108,6 +1120,17 @@ base::optional<MediaView::SharedMediaKey> MediaView::sharedMediaKey() const {
 		| keyForType;
 }
 
+Data::FileOrigin MediaView::fileOrigin() const {
+	if (_msgid) {
+		return _msgid;
+	} else if (_photo && _user) {
+		return Data::FileOriginUserPhoto(_user->bareId(), _photo->id);
+	} else if (_photo && _peer && _peer->userpicPhotoId() == _photo->id) {
+		return Data::FileOriginPeerPhoto(_peer->id);
+	}
+	return Data::FileOrigin();
+}
+
 bool MediaView::validSharedMedia() const {
 	if (auto key = sharedMediaKey()) {
 		if (!_sharedMedia) {
@@ -1447,7 +1470,7 @@ void MediaView::displayPhoto(not_null<PhotoData*> photo, HistoryItem *item) {
 	} else {
 		_from = _user;
 	}
-	_photo->download();
+	_photo->download(fileOrigin());
 	displayFinished();
 }
 
@@ -1488,12 +1511,15 @@ void MediaView::displayDocument(DocumentData *doc, HistoryItem *item) { // empty
 		if (_doc->sticker()) {
 			_doc->checkSticker();
 			if (!_doc->sticker()->img->isNull()) {
-				_current = _doc->sticker()->img->pix();
+				_current = _doc->sticker()->img->pix(fileOrigin());
 			} else {
-				_current = _doc->thumb->pixBlurred(_doc->dimensions.width(), _doc->dimensions.height());
+				_current = _doc->thumb->pixBlurred(
+					fileOrigin(),
+					_doc->dimensions.width(),
+					_doc->dimensions.height());
 			}
 		} else {
-			_doc->automaticLoad(item);
+			_doc->automaticLoad(fileOrigin(), item);
 
 			if (_doc->isAnimation() || _doc->isVideoFile()) {
 				initAnimation();
@@ -1526,7 +1552,7 @@ void MediaView::displayDocument(DocumentData *doc, HistoryItem *item) { // empty
 				_docExtWidth = st::mediaviewFileNameFont->width(_docExt);
 			}
 		} else {
-			_doc->thumb->load();
+			_doc->thumb->load(fileOrigin());
 			int32 tw = _doc->thumb->width(), th = _doc->thumb->height();
 			if (!tw || !th) {
 				_docThumbx = _docThumby = _docThumbw = 0;
@@ -1668,10 +1694,10 @@ void MediaView::initAnimation() {
 	} else if (_doc->dimensions.width() && _doc->dimensions.height()) {
 		auto w = _doc->dimensions.width();
 		auto h = _doc->dimensions.height();
-		_current = _doc->thumb->pixNoCache(w, h, videoThumbOptions(), w / cIntRetinaFactor(), h / cIntRetinaFactor());
+		_current = _doc->thumb->pixNoCache(fileOrigin(), w, h, videoThumbOptions(), w / cIntRetinaFactor(), h / cIntRetinaFactor());
 		if (cRetina()) _current.setDevicePixelRatio(cRetinaFactor());
 	} else {
-		_current = _doc->thumb->pixNoCache(_doc->thumb->width(), _doc->thumb->height(), videoThumbOptions(), st::mediaviewFileIconSize, st::mediaviewFileIconSize);
+		_current = _doc->thumb->pixNoCache(fileOrigin(), _doc->thumb->width(), _doc->thumb->height(), videoThumbOptions(), st::mediaviewFileIconSize, st::mediaviewFileIconSize);
 	}
 }
 
@@ -1684,10 +1710,10 @@ void MediaView::createClipReader() {
 	if (_doc->dimensions.width() && _doc->dimensions.height()) {
 		int w = _doc->dimensions.width();
 		int h = _doc->dimensions.height();
-		_current = _doc->thumb->pixNoCache(w, h, videoThumbOptions(), w / cIntRetinaFactor(), h / cIntRetinaFactor());
+		_current = _doc->thumb->pixNoCache(fileOrigin(), w, h, videoThumbOptions(), w / cIntRetinaFactor(), h / cIntRetinaFactor());
 		if (cRetina()) _current.setDevicePixelRatio(cRetinaFactor());
 	} else {
-		_current = _doc->thumb->pixNoCache(_doc->thumb->width(), _doc->thumb->height(), videoThumbOptions(), st::mediaviewFileIconSize, st::mediaviewFileIconSize);
+		_current = _doc->thumb->pixNoCache(fileOrigin(), _doc->thumb->width(), _doc->thumb->height(), videoThumbOptions(), st::mediaviewFileIconSize, st::mediaviewFileIconSize);
 	}
 	auto mode = (_doc->isVideoFile() || _doc->isVideoMessage())
 		? Media::Clip::Reader::Mode::Video
@@ -1952,20 +1978,20 @@ void MediaView::paintEvent(QPaintEvent *e) {
 		int32 w = _width * cIntRetinaFactor();
 		if (_full <= 0 && _photo->loaded()) {
 			int32 h = int((_photo->full->height() * (qreal(w) / qreal(_photo->full->width()))) + 0.9999);
-			_current = _photo->full->pixNoCache(w, h, Images::Option::Smooth);
+			_current = _photo->full->pixNoCache(fileOrigin(), w, h, Images::Option::Smooth);
 			if (cRetina()) _current.setDevicePixelRatio(cRetinaFactor());
 			_full = 1;
 		} else if (_full < 0 && _photo->medium->loaded()) {
 			int32 h = int((_photo->full->height() * (qreal(w) / qreal(_photo->full->width()))) + 0.9999);
-			_current = _photo->medium->pixNoCache(w, h, Images::Option::Smooth | Images::Option::Blurred);
+			_current = _photo->medium->pixNoCache(fileOrigin(), w, h, Images::Option::Smooth | Images::Option::Blurred);
 			if (cRetina()) _current.setDevicePixelRatio(cRetinaFactor());
 			_full = 0;
 		} else if (_current.isNull() && _photo->thumb->loaded()) {
 			int32 h = int((_photo->full->height() * (qreal(w) / qreal(_photo->full->width()))) + 0.9999);
-			_current = _photo->thumb->pixNoCache(w, h, Images::Option::Smooth | Images::Option::Blurred);
+			_current = _photo->thumb->pixNoCache(fileOrigin(), w, h, Images::Option::Smooth | Images::Option::Blurred);
 			if (cRetina()) _current.setDevicePixelRatio(cRetinaFactor());
 		} else if (_current.isNull()) {
-			_current = _photo->thumb->pix();
+			_current = _photo->thumb->pix(fileOrigin());
 		}
 	}
 	p.setOpacity(1);
@@ -2066,7 +2092,7 @@ void MediaView::paintEvent(QPaintEvent *e) {
 					}
 				} else {
 					int32 rf(cIntRetinaFactor());
-					p.drawPixmap(_docIconRect.topLeft(), _doc->thumb->pix(_docThumbw), QRect(_docThumbx * rf, _docThumby * rf, st::mediaviewFileIconSize * rf, st::mediaviewFileIconSize * rf));
+					p.drawPixmap(_docIconRect.topLeft(), _doc->thumb->pix(fileOrigin(), _docThumbw), QRect(_docThumbx * rf, _docThumby * rf, st::mediaviewFileIconSize * rf, st::mediaviewFileIconSize * rf));
 				}
 
 				paintDocRadialLoading(p, radial, radialOpacity);
@@ -2566,13 +2592,13 @@ void MediaView::preloadData(int delta) {
 	for (auto index = from; index != till; ++index) {
 		auto entity = entityByIndex(index);
 		if (auto photo = base::get_if<not_null<PhotoData*>>(&entity.data)) {
-			(*photo)->download();
+			(*photo)->download(fileOrigin());
 		} else if (auto document = base::get_if<not_null<DocumentData*>>(&entity.data)) {
 			if (auto sticker = (*document)->sticker()) {
-				sticker->img->load();
+				sticker->img->load(fileOrigin());
 			} else {
-				(*document)->thumb->load();
-				(*document)->automaticLoad(entity.item);
+				(*document)->thumb->load(fileOrigin());
+				(*document)->automaticLoad(fileOrigin(), entity.item);
 			}
 		}
 	}
@@ -2847,7 +2873,7 @@ void MediaView::contextMenuEvent(QContextMenuEvent *e) {
 			_menu->deleteLater();
 			_menu = 0;
 		}
-		_menu = new Ui::PopupMenu(nullptr, st::mediaviewPopupMenu);
+		_menu = new Ui::PopupMenu(this, st::mediaviewPopupMenu);
 		updateActions();
 		for_const (auto &action, _actions) {
 			_menu->addAction(action.text, this, action.member);
