@@ -14,7 +14,11 @@ class InputField;
 class ScrollArea;
 class FadeShadow;
 class PlainShadow;
+class FlatLabel;
 class RoundButton;
+class VerticalLayout;
+template <typename Widget>
+class SlideWrap;
 } // namespace Ui
 
 namespace Info {
@@ -30,28 +34,44 @@ struct ValueMap;
 struct ScanInfo;
 class EditScans;
 class PanelDetailsRow;
-enum class SpecialFile;
+enum class FileType;
 enum class PanelDetailsType;
+struct ScanListData;
 
 struct EditDocumentScheme {
 	enum class ValueClass {
 		Fields,
+		Additional,
 		Scans,
 	};
+	enum class AdditionalVisibility {
+		Hidden,
+		OnlyIfError,
+		Shown,
+	};
 	struct Row {
+		using Validator = Fn<base::optional<QString>(const QString &value)>;
+		using Formatter = Fn<QString(const QString &value)>;
 		ValueClass valueClass = ValueClass::Fields;
 		PanelDetailsType inputType = PanelDetailsType();
 		QString key;
 		QString label;
-		Fn<base::optional<QString>(const QString &value)> error;
-		Fn<QString(const QString &value)> format;
+		Validator error;
+		Formatter format;
 		int lengthLimit = 0;
-		QString keyForAttachmentTo; // attach last_name to first_name
+		QString keyForAttachmentTo; // Attach [last|middle]_name to first_*.
+		QString additionalFallbackKey; // *_name_native from *_name.
 	};
 	std::vector<Row> rows;
-	QString rowsHeader;
+	QString fieldsHeader;
+	QString detailsHeader;
 	QString scansHeader;
 
+	QString additionalDependencyKey;
+	Fn<AdditionalVisibility(const QString &dependency)> additionalShown;
+	Fn<QString(const QString &dependency)> additionalHeader;
+	Fn<QString(const QString &dependency)> additionalDescription;
+
 };
 
 class PanelEditDocument : public Ui::RpWidget {
@@ -62,15 +82,27 @@ public:
 		QWidget *parent,
 		not_null<PanelController*> controller,
 		Scheme scheme,
+		const QString &error,
 		const ValueMap &data,
-		const ValueMap &scanData,
-		const QString &missingScansError,
-		std::vector<ScanInfo> &&files,
-		std::map<SpecialFile, ScanInfo> &&specialFiles);
+		const QString &scansError,
+		const ValueMap &scansData,
+		ScanListData &&scans,
+		base::optional<ScanListData> &&translations,
+		std::map<FileType, ScanInfo> &&specialFiles);
 	PanelEditDocument(
 		QWidget *parent,
 		not_null<PanelController*> controller,
 		Scheme scheme,
+		const QString &scansError,
+		const ValueMap &scansData,
+		ScanListData &&scans,
+		base::optional<ScanListData> &&translations,
+		std::map<FileType, ScanInfo> &&specialFiles);
+	PanelEditDocument(
+		QWidget *parent,
+		not_null<PanelController*> controller,
+		Scheme scheme,
+		const QString &error,
 		const ValueMap &data);
 
 	bool hasUnsavedChanges() const;
@@ -82,23 +114,37 @@ protected:
 private:
 	struct Result;
 	void setupControls(
-		const ValueMap &data,
-		const ValueMap *scanData,
-		const QString &missingScansError,
-		std::vector<ScanInfo> &&files,
-		std::map<SpecialFile, ScanInfo> &&specialFiles);
+		const QString *error,
+		const ValueMap *data,
+		const QString *scansError,
+		const ValueMap *scansData,
+		ScanListData &&scans,
+		base::optional<ScanListData> &&translations,
+		std::map<FileType, ScanInfo> &&specialFiles);
 	not_null<Ui::RpWidget*> setupContent(
-		const ValueMap &data,
-		const ValueMap *scanData,
-		const QString &missingScansError,
-		std::vector<ScanInfo> &&files,
-		std::map<SpecialFile, ScanInfo> &&specialFiles);
+		const QString *error,
+		const ValueMap *data,
+		const QString *scansError,
+		const ValueMap *scansData,
+		ScanListData &&scans,
+		base::optional<ScanListData> &&translations,
+		std::map<FileType, ScanInfo> &&specialFiles);
 	void updateControlsGeometry();
+	void updateCommonError();
 
 	Result collect() const;
+	void fillAdditionalFromFallbacks(Result &result) const;
 	bool validate();
 	void save();
 
+	void createDetailsRow(
+		not_null<Ui::VerticalLayout*> container,
+		int i,
+		const Scheme::Row &row,
+		const ValueMap &fields,
+		int maxLabelWidth);
+	not_null<PanelDetailsRow*> findRow(const QString &key) const;
+
 	not_null<PanelController*> _controller;
 	Scheme _scheme;
 
@@ -107,7 +153,10 @@ private:
 	object_ptr<Ui::PlainShadow> _bottomShadow;
 
 	QPointer<EditScans> _editScans;
+	QPointer<Ui::SlideWrap<Ui::FlatLabel>> _commonError;
 	std::map<int, QPointer<PanelDetailsRow>> _details;
+	bool _fieldsChanged = false;
+	bool _additionalShown = false;
 
 	QPointer<Info::Profile::Button> _delete;
 
