@@ -247,9 +247,6 @@ GtkDialog *QGtkDialog::gtkDialog() const {
 }
 
 void QGtkDialog::exec() {
-	if (auto w = App::wnd()) {
-		w->reActivateWindow();
-	}
 	if (modality() == Qt::ApplicationModal) {
 		// block input to the whole app, including other GTK dialogs
 		Libs::gtk_dialog_run(gtkDialog());
@@ -413,6 +410,12 @@ int GtkFileDialog::exec() {
 
 	show();
 
+	if (const auto parent = parentWidget()) {
+		App::CallDelayed(200, parent, [=] {
+			parent->activateWindow();
+		});
+	}
+
 	QPointer<QDialog> guard = this;
 	d->exec();
 	if (guard.isNull())
