@@ -29,6 +29,8 @@ namespace {
 
 constexpr auto kIgnoreActivationTimeoutMs = 500;
 
+base::optional<bool> ApplicationIsActive;
+
 } // namespace
 
 using Platform::Q2NSString;
@@ -85,6 +87,7 @@ using Platform::NS2QString;
 - (BOOL) applicationShouldHandleReopen:(NSApplication *)theApplication hasVisibleWindows:(BOOL)flag;
 - (void) applicationDidFinishLaunching:(NSNotification *)aNotification;
 - (void) applicationDidBecomeActive:(NSNotification *)aNotification;
+- (void) applicationDidResignActive:(NSNotification *)aNotification;
 - (void) receiveWakeNote:(NSNotification*)note;
 
 - (void) setWatchingMediaKeys:(bool)watching;
@@ -126,6 +129,7 @@ ApplicationDelegate *_sharedDelegate = nil;
 }
 
 - (void) applicationDidBecomeActive:(NSNotification *)aNotification {
+	ApplicationIsActive = true;
 	if (auto messenger = Messenger::InstancePointer()) {
 		if (!_ignoreActivation) {
 			messenger->handleAppActivated();
@@ -138,6 +142,10 @@ ApplicationDelegate *_sharedDelegate = nil;
 	}
 }
 
+- (void) applicationDidResignActive:(NSNotification *)aNotification {
+	ApplicationIsActive = false;
+}
+
 - (void) receiveWakeNote:(NSNotification*)aNotification {
 	if (auto messenger = Messenger::InstancePointer()) {
 		messenger->checkLocalTime();
@@ -187,6 +195,12 @@ void SetWatchingMediaKeys(bool watching) {
 	}
 }
 
+bool IsApplicationActive() {
+	return ApplicationIsActive
+		? *ApplicationIsActive
+		: (static_cast<QApplication*>(QApplication::instance())->activeWindow() != nullptr);
+}
+
 void InitOnTopPanel(QWidget *panel) {
 	Expects(!panel->windowHandle());
 
