@@ -16,7 +16,6 @@ uint64 gRealBetaVersion = AppBetaVersion;
 QByteArray gBetaPrivateKey;
 
 bool gTestMode = false;
-bool gDebug = false;
 bool gManyInstance = false;
 QString gKeyFile;
 QString gWorkingDir, gExeDir, gExeName;
@@ -40,7 +39,6 @@ bool gRestartingUpdate = false, gRestarting = false, gRestartingToSettings = fal
 int32 gLastUpdateCheck = 0;
 bool gNoStartUpdate = false;
 bool gStartToSettings = false;
-bool gReplaceEmojis = true;
 
 bool gCtrlEnter = false;
 
@@ -87,7 +85,6 @@ DBIPlatform gPlatform = dbipLinux32;
 #error Unknown platform
 #endif
 QString gPlatformString;
-QUrl gUpdateURL;
 bool gIsElCapitan = false;
 bool gIsSnowLeopard = false;
 
