@@ -62,13 +62,10 @@ void AdvancedWidget::createControls() {
 	} else {
 		style::margins slidedPadding(0, marginLarge.bottom() / 2, 0, marginLarge.bottom() - (marginLarge.bottom() / 2));
 		createChildRow(_useDefaultTheme, marginLarge, slidedPadding, lang(lng_settings_bg_use_default), SLOT(onUseDefaultTheme()));
-		if (!Window::Theme::IsNonDefaultUsed()) {
+		if (!Window::Theme::SuggestThemeReset()) {
 			_useDefaultTheme->hide(anim::type::instant);
 		}
-		createChildRow(_toggleNightTheme, marginLarge, slidedPadding, getNightThemeToggleText(), SLOT(onToggleNightTheme()));
-		if (Window::Theme::IsNonDefaultUsed()) {
-			_toggleNightTheme->hide(anim::type::instant);
-		}
+		createChildRow(_toggleNightTheme, marginLarge, getNightThemeToggleText(), SLOT(onToggleNightTheme()));
 	}
 	createChildRow(_telegramFAQ, marginLarge, lang(lng_settings_faq), SLOT(onTelegramFAQ()));
 	if (self()) {
@@ -78,14 +75,13 @@ void AdvancedWidget::createControls() {
 }
 
 void AdvancedWidget::checkNonDefaultTheme() {
-	if (self()) return;
+	if (self()) {
+		return;
+	}
 	_useDefaultTheme->toggle(
-		Window::Theme::IsNonDefaultUsed(),
-		anim::type::normal);
-	_toggleNightTheme->entity()->setText(getNightThemeToggleText());
-	_toggleNightTheme->toggle(
-		!Window::Theme::IsNonDefaultUsed(),
+		Window::Theme::SuggestThemeReset(),
 		anim::type::normal);
+	_toggleNightTheme->setText(getNightThemeToggleText());
 }
 
 void AdvancedWidget::onManageLocalStorage() {
@@ -120,7 +116,7 @@ void AdvancedWidget::onUseDefaultTheme() {
 }
 
 void AdvancedWidget::onToggleNightTheme() {
-	Window::Theme::SwitchNightTheme(!Window::Theme::IsNightTheme());
+	Window::Theme::ToggleNightMode();
 }
 
 void AdvancedWidget::onAskQuestion() {
@@ -149,7 +145,9 @@ void AdvancedWidget::supportGot(const MTPhelp_Support &support) {
 }
 
 QString AdvancedWidget::getNightThemeToggleText() const {
-	return lang(Window::Theme::IsNightTheme() ? lng_settings_disable_night_theme : lng_settings_enable_night_theme);
+	return lang(Window::Theme::IsNightMode()
+		? lng_settings_disable_night_theme
+		: lng_settings_enable_night_theme);
 }
 
 void AdvancedWidget::onTelegramFAQ() {
