@@ -9,15 +9,13 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "settings/settings_inner_widget.h"
 #include "settings/settings_fixed_bar.h"
-#include "styles/style_settings.h"
-#include "styles/style_window.h"
-#include "styles/style_boxes.h"
 #include "platform/platform_specific.h"
 #include "ui/widgets/scroll_area.h"
 #include "ui/widgets/buttons.h"
 #include "ui/toast/toast.h"
 #include "mainwindow.h"
 #include "mainwidget.h"
+#include "data/data_session.h"
 #include "storage/localstorage.h"
 #include "boxes/confirm_box.h"
 #include "lang/lang_keys.h"
@@ -30,6 +28,11 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "window/themes/window_theme.h"
 #include "window/themes/window_theme_editor.h"
 #include "media/media_audio_track.h"
+#include "mainwindow.h"
+#include "window/window_controller.h"
+#include "styles/style_settings.h"
+#include "styles/style_window.h"
+#include "styles/style_boxes.h"
 
 namespace Settings {
 namespace {
@@ -47,17 +50,20 @@ void fillCodes() {
 			Messenger::Instance().onSwitchDebugMode();
 		}));
 	});
+	Codes.insert(qsl("viewlogs"), [] {
+		File::ShowInFolder(cWorkingDir() + "log.txt");
+	});
 	Codes.insert(qsl("testmode"), [] {
 		auto text = cTestMode() ? qsl("Do you want to disable TEST mode?") : qsl("Do you want to enable TEST mode?\n\nYou will be switched to test cloud.");
 		Ui::show(Box<ConfirmBox>(text, [] {
 			Messenger::Instance().onSwitchTestMode();
 		}));
 	});
-#ifndef TDESKTOP_DISABLE_AUTOUPDATE
-	Codes.insert(qsl("testupdate"), [] {
-		Core::UpdateChecker().test();
-	});
-#endif // TDESKTOP_DISABLE_AUTOUPDATE
+	if (!Core::UpdaterDisabled()) {
+		Codes.insert(qsl("testupdate"), [] {
+			Core::UpdateChecker().test();
+		});
+	}
 	Codes.insert(qsl("loadlang"), [] {
 		Lang::CurrentCloudManager().switchToLanguage(qsl("custom"));
 	});
@@ -125,6 +131,9 @@ void fillCodes() {
 		Platform::RegisterCustomScheme();
 		Ui::Toast::Show("Forced custom scheme register.");
 	});
+	Codes.insert(qsl("export"), [] {
+		Auth().data().startExport();
+	});
 
 	auto audioFilters = qsl("Audio files (*.wav *.mp3);;") + FileDialog::AllFilesFilter();
 	auto audioKeys = {
