@@ -14,6 +14,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "storage/localstorage.h"
 #include "platform/platform_file_utilities.h"
 #include "auth_session.h"
+#include "apiwrap.h"
 #include "core/crash_reports.h"
 #include "base/bytes.h"
 #include "base/openssl_help.h"
@@ -392,10 +393,22 @@ void FileLoader::startLoading(bool loadFirst, bool prior) {
 	loadPart();
 }
 
-mtpFileLoader::mtpFileLoader(const StorageImageLocation *location, int32 size, LoadFromCloudSetting fromCloud, bool autoLoading)
-: FileLoader(QString(), size, UnknownFileLocation, LoadToCacheAsWell, fromCloud, autoLoading)
+mtpFileLoader::mtpFileLoader(
+	not_null<StorageImageLocation*> location,
+	Data::FileOrigin origin,
+	int32 size,
+	LoadFromCloudSetting fromCloud
+	, bool autoLoading)
+: FileLoader(
+	QString(),
+	size,
+	UnknownFileLocation,
+	LoadToCacheAsWell,
+	fromCloud,
+	autoLoading)
 , _dcId(location->dc())
-, _location(location) {
+, _location(location)
+, _origin(origin) {
 	auto shiftedDcId = MTP::downloadDcId(_dcId, 0);
 	auto i = queues.find(shiftedDcId);
 	if (i == queues.cend()) {
@@ -404,12 +417,30 @@ mtpFileLoader::mtpFileLoader(const StorageImageLocation *location, int32 size, L
 	_queue = &i.value();
 }
 
-mtpFileLoader::mtpFileLoader(int32 dc, uint64 id, uint64 accessHash, int32 version, LocationType type, const QString &to, int32 size, LoadToCacheSetting toCache, LoadFromCloudSetting fromCloud, bool autoLoading)
-: FileLoader(to, size, type, toCache, fromCloud, autoLoading)
+mtpFileLoader::mtpFileLoader(
+	int32 dc,
+	uint64 id,
+	uint64 accessHash,
+	const QByteArray &fileReference,
+	Data::FileOrigin origin,
+	LocationType type,
+	const QString &to,
+	int32 size,
+	LoadToCacheSetting toCache,
+	LoadFromCloudSetting fromCloud,
+	bool autoLoading)
+: FileLoader(
+	to,
+	size,
+	type,
+	toCache,
+	fromCloud,
+	autoLoading)
 , _dcId(dc)
 , _id(id)
 , _accessHash(accessHash)
-, _version(version) {
+, _fileReference(fileReference)
+, _origin(origin) {
 	auto shiftedDcId = MTP::downloadDcId(_dcId, 0);
 	auto i = queues.find(shiftedDcId);
 	if (i == queues.cend()) {
@@ -418,8 +449,18 @@ mtpFileLoader::mtpFileLoader(int32 dc, uint64 id, uint64 accessHash, int32 versi
 	_queue = &i.value();
 }
 
-mtpFileLoader::mtpFileLoader(const WebFileLocation *location, int32 size, LoadFromCloudSetting fromCloud, bool autoLoading)
-: FileLoader(QString(), size, UnknownFileLocation, LoadToCacheAsWell, fromCloud, autoLoading)
+mtpFileLoader::mtpFileLoader(
+	const WebFileLocation *location,
+	int32 size,
+	LoadFromCloudSetting fromCloud,
+	bool autoLoading)
+: FileLoader(
+	QString(),
+	size,
+	UnknownFileLocation,
+	LoadToCacheAsWell,
+	fromCloud,
+	autoLoading)
 , _dcId(location->dc())
 , _urlLocation(location) {
 	auto shiftedDcId = MTP::downloadDcId(_dcId, 0);
@@ -434,6 +475,38 @@ int32 mtpFileLoader::currentOffset(bool includeSkipped) const {
 	return (_fileIsOpen ? _file.size() : _data.size()) - (includeSkipped ? 0 : _skippedBytes);
 }
 
+Data::FileOrigin mtpFileLoader::fileOrigin() const {
+	return _origin;
+}
+
+void mtpFileLoader::refreshFileReferenceFrom(
+		const Data::UpdatedFileReferences &data,
+		int requestId,
+		const QByteArray &current) {
+	const auto updated = [&] {
+		if (_location) {
+			const auto i = data.find(Data::SimpleFileLocationId(
+				_location->volume(),
+				_location->dc(),
+				_location->local()));
+			return (i == end(data)) ? QByteArray() : i->second;
+		}
+		const auto i = data.find(_id);
+		return (i == end(data)) ? QByteArray() : i->second;
+	}();
+	if (updated.isEmpty() || updated == current) {
+		cancel(true);
+		return;
+	}
+	if (_location) {
+		_location->refreshFileReference(updated);
+	} else {
+		_fileReference = updated;
+	}
+	const auto offset = finishSentRequestGetOffset(requestId);
+	makeRequest(offset);
+}
+
 bool mtpFileLoader::loadPart() {
 	if (_finished || _lastComplete || (!_sentRequests.empty() && !_size)) {
 		return false;
@@ -480,13 +553,39 @@ void mtpFileLoader::makeRequest(int offset) {
 		auto shiftedDcId = MTP::downloadDcId(requestData.dcId, requestData.dcIndex);
 		if (_cdnDcId) {
 			Assert(requestData.dcId == _cdnDcId);
-			return MTP::send(MTPupload_GetCdnFile(MTP_bytes(_cdnToken), MTP_int(offset), MTP_int(limit)), rpcDone(&mtpFileLoader::cdnPartLoaded), rpcFail(&mtpFileLoader::cdnPartFailed), shiftedDcId, 50);
+			return MTP::send(
+				MTPupload_GetCdnFile(
+					MTP_bytes(_cdnToken),
+					MTP_int(offset),
+					MTP_int(limit)),
+				rpcDone(&mtpFileLoader::cdnPartLoaded),
+				rpcFail(&mtpFileLoader::cdnPartFailed),
+				shiftedDcId,
+				50);
 		} else if (_urlLocation) {
 			Assert(requestData.dcId == _dcId);
-			return MTP::send(MTPupload_GetWebFile(MTP_inputWebFileLocation(MTP_bytes(_urlLocation->url()), MTP_long(_urlLocation->accessHash())), MTP_int(offset), MTP_int(limit)), rpcDone(&mtpFileLoader::webPartLoaded), rpcFail(&mtpFileLoader::partFailed), shiftedDcId, 50);
+			return MTP::send(
+				MTPupload_GetWebFile(
+					MTP_inputWebFileLocation(
+						MTP_bytes(_urlLocation->url()),
+						MTP_long(_urlLocation->accessHash())),
+					MTP_int(offset),
+					MTP_int(limit)),
+				rpcDone(&mtpFileLoader::webPartLoaded),
+				rpcFail(&mtpFileLoader::partFailed),
+				shiftedDcId,
+				50);
 		} else {
 			Assert(requestData.dcId == _dcId);
-			return MTP::send(MTPupload_GetFile(computeLocation(), MTP_int(offset), MTP_int(limit)), rpcDone(&mtpFileLoader::normalPartLoaded), rpcFail(&mtpFileLoader::partFailed), shiftedDcId, 50);
+			return MTP::send(
+				MTPupload_GetFile(
+					computeLocation(),
+					MTP_int(offset),
+					MTP_int(limit)),
+				rpcDone(&mtpFileLoader::normalPartLoaded),
+				rpcFail(&mtpFileLoader::partFailed),
+				shiftedDcId,
+				50);
 		}
 	};
 	placeSentRequest(send(), requestData);
@@ -494,11 +593,20 @@ void mtpFileLoader::makeRequest(int offset) {
 
 MTPInputFileLocation mtpFileLoader::computeLocation() const {
 	if (_location) {
-		return MTP_inputFileLocation(MTP_long(_location->volume()), MTP_int(_location->local()), MTP_long(_location->secret()));
+		return MTP_inputFileLocation(
+			MTP_long(_location->volume()),
+			MTP_int(_location->local()),
+			MTP_long(_location->secret()),
+			MTP_bytes(_location->fileReference()));
 	} else if (_locationType == SecureFileLocation) {
-		return MTP_inputSecureFileLocation(MTP_long(_id), MTP_long(_accessHash));
+		return MTP_inputSecureFileLocation(
+			MTP_long(_id),
+			MTP_long(_accessHash));
 	}
-	return MTP_inputDocumentFileLocation(MTP_long(_id), MTP_long(_accessHash), MTP_int(_version));
+	return MTP_inputDocumentFileLocation(
+		MTP_long(_id),
+		MTP_long(_accessHash),
+		MTP_bytes(_fileReference));
 }
 
 void mtpFileLoader::requestMoreCdnFileHashes() {
@@ -524,7 +632,9 @@ void mtpFileLoader::requestMoreCdnFileHashes() {
 	placeSentRequest(requestId, requestData);
 }
 
-void mtpFileLoader::normalPartLoaded(const MTPupload_File &result, mtpRequestId requestId) {
+void mtpFileLoader::normalPartLoaded(
+		const MTPupload_File &result,
+		mtpRequestId requestId) {
 	Expects(!_finished);
 	Expects(result.type() == mtpc_upload_fileCdnRedirect || result.type() == mtpc_upload_file);
 
@@ -536,7 +646,9 @@ void mtpFileLoader::normalPartLoaded(const MTPupload_File &result, mtpRequestId
 	return partLoaded(offset, buffer);
 }
 
-void mtpFileLoader::webPartLoaded(const MTPupload_WebFile &result, mtpRequestId requestId) {
+void mtpFileLoader::webPartLoaded(
+		const MTPupload_WebFile &result,
+		mtpRequestId requestId) {
 	Expects(result.type() == mtpc_upload_webFile);
 
 	auto offset = finishSentRequestGetOffset(requestId);
@@ -682,7 +794,7 @@ void mtpFileLoader::placeSentRequest(mtpRequestId requestId, const RequestData &
 
 int mtpFileLoader::finishSentRequestGetOffset(mtpRequestId requestId) {
 	auto it = _sentRequests.find(requestId);
-	Expects(it != _sentRequests.cend());
+	Assert(it != _sentRequests.cend());
 
 	auto requestData = it->second;
 	_downloader->requestedAmountIncrement(requestData.dcId, requestData.dcIndex, -partSize());
@@ -764,7 +876,7 @@ bool mtpFileLoader::feedPart(int offset, bytes::const_span buffer) {
 			if (_urlLocation) {
 				Local::writeImage(storageKey(*_urlLocation), StorageImageSaved(_data));
 			} else if (_locationType != UnknownFileLocation) { // audio, video, document
-				auto mkey = mediaKey(_locationType, _dcId, _id, _version);
+				auto mkey = mediaKey(_locationType, _dcId, _id);
 				if (!_filename.isEmpty()) {
 					Local::writeFileLocation(mkey, FileLocation(_filename));
 				}
@@ -799,25 +911,48 @@ void mtpFileLoader::partLoaded(int offset, bytes::const_span buffer) {
 	}
 }
 
-bool mtpFileLoader::partFailed(const RPCError &error) {
-	if (MTP::isDefaultHandledError(error)) return false;
-
+bool mtpFileLoader::partFailed(
+		const RPCError &error,
+		mtpRequestId requestId) {
+	if (MTP::isDefaultHandledError(error)) {
+		return false;
+	}
+	if (error.code() == 400
+		&& error.type().startsWith(qstr("FILE_REFERENCE_"))) {
+		Auth().api().refreshFileReference(
+			_origin,
+			this,
+			requestId,
+			_location ? _location->fileReference() : _fileReference);
+		return true;
+	}
 	cancel(true);
 	return true;
 }
 
-bool mtpFileLoader::cdnPartFailed(const RPCError &error, mtpRequestId requestId) {
-	if (MTP::isDefaultHandledError(error)) return false;
+bool mtpFileLoader::cdnPartFailed(
+		const RPCError &error,
+		mtpRequestId requestId) {
+	if (MTP::isDefaultHandledError(error)) {
+		return false;
+	}
 
 	if (requestId == _cdnHashesRequestId) {
 		_cdnHashesRequestId = 0;
 	}
-	if (error.type() == qstr("FILE_TOKEN_INVALID") || error.type() == qstr("REQUEST_TOKEN_INVALID")) {
+	if (error.type() == qstr("FILE_TOKEN_INVALID")
+		|| error.type() == qstr("REQUEST_TOKEN_INVALID")) {
 		auto offset = finishSentRequestGetOffset(requestId);
-		changeCDNParams(offset, 0, QByteArray(), QByteArray(), QByteArray(), QVector<MTPFileHash>());
+		changeCDNParams(
+			offset,
+			0,
+			QByteArray(),
+			QByteArray(),
+			QByteArray(),
+			QVector<MTPFileHash>());
 		return true;
 	}
-	return partFailed(error);
+	return partFailed(error, requestId);
 }
 
 void mtpFileLoader::cancelRequests() {
@@ -850,8 +985,16 @@ void mtpFileLoader::addCdnHashes(const QVector<MTPFileHash> &hashes) {
 	}
 }
 
-void mtpFileLoader::changeCDNParams(int offset, MTP::DcId dcId, const QByteArray &token, const QByteArray &encryptionKey, const QByteArray &encryptionIV, const QVector<MTPFileHash> &hashes) {
-	if (dcId != 0 && (encryptionKey.size() != MTP::CTRState::KeySize || encryptionIV.size() != MTP::CTRState::IvecSize)) {
+void mtpFileLoader::changeCDNParams(
+		int offset,
+		MTP::DcId dcId,
+		const QByteArray &token,
+		const QByteArray &encryptionKey,
+		const QByteArray &encryptionIV,
+		const QVector<MTPFileHash> &hashes) {
+	if (dcId != 0
+		&& (encryptionKey.size() != MTP::CTRState::KeySize
+			|| encryptionIV.size() != MTP::CTRState::IvecSize)) {
 		LOG(("Message Error: Wrong key (%1) / iv (%2) size in CDN params").arg(encryptionKey.size()).arg(encryptionIV.size()));
 		cancel(true);
 		return;
@@ -897,7 +1040,7 @@ bool mtpFileLoader::tryLoadLocal() {
 		_localTaskId = Local::startImageLoad(storageKey(*_location), this);
 	} else {
 		if (_toCache == LoadToCacheAsWell) {
-			MediaKey mkey = mediaKey(_locationType, _dcId, _id, _version);
+			MediaKey mkey = mediaKey(_locationType, _dcId, _id);
 			if (_locationType == DocumentFileLocation) {
 				_localTaskId = Local::startStickerImageLoad(mkey, this);
 			} else if (_locationType == AudioFileLocation) {
