@@ -22,7 +22,7 @@ TextParseOptions _checkboxOptions = {
 
 } // namespace
 
-AbstractCheckView::AbstractCheckView(int duration, bool checked, base::lambda<void()> updateCallback)
+AbstractCheckView::AbstractCheckView(int duration, bool checked, Fn<void()> updateCallback)
 : _duration(duration)
 , _checked(checked)
 , _updateCallback(std::move(updateCallback)) {
@@ -36,13 +36,19 @@ void AbstractCheckView::setCheckedFast(bool checked) {
 	}
 }
 
-void AbstractCheckView::setUpdateCallback(base::lambda<void()> updateCallback) {
+void AbstractCheckView::setUpdateCallback(Fn<void()> updateCallback) {
 	_updateCallback = std::move(updateCallback);
 	if (_toggleAnimation.animating()) {
 		_toggleAnimation.setUpdateCallback(_updateCallback);
 	}
 }
 
+void AbstractCheckView::update() {
+	if (_updateCallback) {
+		_updateCallback();
+	}
+}
+
 void AbstractCheckView::setCheckedAnimated(bool checked) {
 	if (_checked != checked) {
 		_checked = checked;
@@ -61,7 +67,7 @@ float64 AbstractCheckView::currentAnimationValue(TimeMs ms) {
 ToggleView::ToggleView(
 	const style::Toggle &st,
 	bool checked,
-	base::lambda<void()> updateCallback)
+	Fn<void()> updateCallback)
 : AbstractCheckView(st.duration, checked, std::move(updateCallback))
 , _st(&st) {
 }
@@ -198,7 +204,7 @@ bool ToggleView::checkRippleStartPosition(QPoint position) const {
 	return QRect(QPoint(0, 0), rippleSize()).contains(position);
 }
 
-CheckView::CheckView(const style::Check &st, bool checked, base::lambda<void()> updateCallback) : AbstractCheckView(st.duration, checked, std::move(updateCallback))
+CheckView::CheckView(const style::Check &st, bool checked, Fn<void()> updateCallback) : AbstractCheckView(st.duration, checked, std::move(updateCallback))
 , _st(&st) {
 }
 
@@ -212,10 +218,17 @@ void CheckView::setStyle(const style::Check &st) {
 
 void CheckView::paint(Painter &p, int left, int top, int outerWidth, TimeMs ms) {
 	auto toggled = currentAnimationValue(ms);
-	auto pen = anim::pen(_st->untoggledFg, _st->toggledFg, toggled);
+	auto pen = _untoggledOverride
+		? anim::pen(*_untoggledOverride, _st->toggledFg, toggled)
+		: anim::pen(_st->untoggledFg, _st->toggledFg, toggled);
 	pen.setWidth(_st->thickness);
 	p.setPen(pen);
-	p.setBrush(anim::brush(_st->bg, anim::color(_st->untoggledFg, _st->toggledFg, toggled), toggled));
+	p.setBrush(anim::brush(
+		_st->bg,
+		(_untoggledOverride
+			? anim::color(*_untoggledOverride, _st->toggledFg, toggled)
+			: anim::color(_st->untoggledFg, _st->toggledFg, toggled)),
+		toggled));
 
 	{
 		PainterHighQualityEnabler hq(p);
@@ -239,7 +252,17 @@ bool CheckView::checkRippleStartPosition(QPoint position) const {
 	return QRect(QPoint(0, 0), rippleSize()).contains(position);
 }
 
-RadioView::RadioView(const style::Radio &st, bool checked, base::lambda<void()> updateCallback) : AbstractCheckView(st.duration, checked, std::move(updateCallback))
+void CheckView::setUntoggledOverride(
+		base::optional<QColor> untoggledOverride) {
+	_untoggledOverride = untoggledOverride;
+	update();
+}
+
+RadioView::RadioView(
+	const style::Radio &st,
+	bool checked,
+	Fn<void()> updateCallback)
+: AbstractCheckView(st.duration, checked, std::move(updateCallback))
 , _st(&st) {
 }
 
@@ -255,7 +278,9 @@ void RadioView::paint(Painter &p, int left, int top, int outerWidth, TimeMs ms)
 	PainterHighQualityEnabler hq(p);
 
 	auto toggled = currentAnimationValue(ms);
-	auto pen = anim::pen(_st->untoggledFg, _st->toggledFg, toggled);
+	auto pen = _untoggledOverride
+		? anim::pen(*_untoggledOverride, _st->toggledFg, toggled)
+		: anim::pen(_st->untoggledFg, _st->toggledFg, toggled);
 	pen.setWidth(_st->thickness);
 	p.setPen(pen);
 	p.setBrush(_st->bg);
@@ -265,7 +290,9 @@ void RadioView::paint(Painter &p, int left, int top, int outerWidth, TimeMs ms)
 
 	if (toggled > 0) {
 		p.setPen(Qt::NoPen);
-		p.setBrush(anim::brush(_st->untoggledFg, _st->toggledFg, toggled));
+		p.setBrush(_untoggledOverride
+			? anim::brush(*_untoggledOverride, _st->toggledFg, toggled)
+			: anim::brush(_st->untoggledFg, _st->toggledFg, toggled));
 
 		auto skip0 = _st->diameter / 2., skip1 = _st->skip / 10., checkSkip = skip0 * (1. - toggled) + skip1 * toggled;
 		p.drawEllipse(rtlrect(QRectF(left, top, _st->diameter, _st->diameter).marginsRemoved(QMarginsF(checkSkip, checkSkip, checkSkip, checkSkip)), outerWidth));
@@ -295,16 +322,52 @@ bool RadioView::checkRippleStartPosition(QPoint position) const {
 	return QRect(QPoint(0, 0), rippleSize()).contains(position);
 }
 
-Checkbox::Checkbox(QWidget *parent, const QString &text, bool checked, const style::Checkbox &st, const style::Check &checkSt) : Checkbox(parent, text, st, std::make_unique<CheckView>(checkSt, checked, [this] { updateCheck(); })) {
+void RadioView::setUntoggledOverride(
+		base::optional<QColor> untoggledOverride) {
+	_untoggledOverride = untoggledOverride;
+	update();
 }
 
-Checkbox::Checkbox(QWidget *parent, const QString &text, bool checked, const style::Checkbox &st, const style::Toggle &toggleSt) : Checkbox(parent, text, st, std::make_unique<ToggleView>(toggleSt, checked, [this] { updateCheck(); })) {
+Checkbox::Checkbox(
+	QWidget *parent,
+	const QString &text,
+	bool checked,
+	const style::Checkbox &st,
+	const style::Check &checkSt)
+: Checkbox(
+	parent,
+	text,
+	st,
+	std::make_unique<CheckView>(
+		checkSt,
+		checked)) {
 }
 
-Checkbox::Checkbox(QWidget *parent, const QString &text, const style::Checkbox &st, std::unique_ptr<AbstractCheckView> check) : RippleButton(parent, st.ripple)
+Checkbox::Checkbox(
+	QWidget *parent,
+	const QString &text,
+	bool checked,
+	const style::Checkbox &st,
+	const style::Toggle &toggleSt)
+: Checkbox(
+	parent,
+	text,
+	st,
+	std::make_unique<ToggleView>(
+		toggleSt,
+		checked)) {
+}
+
+Checkbox::Checkbox(
+	QWidget *parent,
+	const QString &text,
+	const style::Checkbox &st,
+	std::unique_ptr<AbstractCheckView> check)
+: RippleButton(parent, st.ripple)
 , _st(st)
 , _check(std::move(check))
 , _text(_st.style, text, _checkboxOptions) {
+	_check->setUpdateCallback([=] { updateCheck(); });
 	resizeToText();
 	setCursor(style::cur_pointer);
 }
@@ -502,9 +565,39 @@ void RadiobuttonGroup::setValue(int value) {
 	}
 }
 
-Radiobutton::Radiobutton(QWidget *parent, const std::shared_ptr<RadiobuttonGroup> &group, int value, const QString &text, const style::Checkbox &st, const style::Radio &radioSt) : Checkbox(parent, text, st, std::make_unique<RadioView>(radioSt, (group->hasValue() && group->value() == value), [this] { updateCheck(); }))
+Radiobutton::Radiobutton(
+	QWidget *parent,
+	const std::shared_ptr<RadiobuttonGroup> &group,
+	int value,
+	const QString &text,
+	const style::Checkbox &st,
+	const style::Radio &radioSt)
+: Radiobutton(
+	parent,
+	group,
+	value,
+	text,
+	st,
+	std::make_unique<RadioView>(
+		radioSt,
+		(group->hasValue() && group->value() == value))) {
+}
+
+Radiobutton::Radiobutton(
+	QWidget *parent,
+	const std::shared_ptr<RadiobuttonGroup> &group,
+	int value,
+	const QString &text,
+	const style::Checkbox &st,
+	std::unique_ptr<AbstractCheckView> check)
+: Checkbox(
+	parent,
+	text,
+	st,
+	std::move(check))
 , _group(group)
 , _value(value) {
+	checkbox()->setChecked(group->hasValue() && group->value() == value);
 	_group->registerButton(this);
 	subscribe(checkbox()->checkedChanged, [this](bool checked) {
 		if (checked) {
