@@ -342,6 +342,9 @@ void Filler::addUserActions(not_null<UserData*> user) {
 				lang(lng_profile_invite_to_group),
 				[user] { AddBotToGroupBoxController::Start(user); });
 		}
+		_addAction(
+			lang(lng_profile_export_chat),
+			[=] { PeerMenuExportChat(user); });
 	}
 	_addAction(
 		lang(lng_profile_delete_conversation),
@@ -364,6 +367,9 @@ void Filler::addChatActions(not_null<ChatData*> chat) {
 				lang(lng_profile_add_participant),
 				[chat] { AddChatMembers(chat); });
 		}
+		_addAction(
+			lang(lng_profile_export_chat),
+			[=] { PeerMenuExportChat(chat); });
 	}
 	_addAction(
 		lang(lng_profile_clear_and_exit),
@@ -398,6 +404,11 @@ void Filler::addChannelActions(not_null<ChannelData*> channel) {
 				lang(lng_channel_add_members),
 				[channel] { PeerMenuAddChannelMembers(channel); });
 		}
+		_addAction(
+			lang(isGroup
+				? lng_profile_export_chat
+				: lng_profile_export_channel),
+			[=] { PeerMenuExportChat(channel); });
 	}
 	if (channel->amIn()) {
 		if (isGroup && !channel->isPublic()) {
@@ -542,6 +553,10 @@ void FeedFiller::addUngroup() {
 
 } // namespace
 
+void PeerMenuExportChat(not_null<PeerData*> peer) {
+	Auth().data().startExport(peer);
+}
+
 void PeerMenuDeleteContact(not_null<UserData*> user) {
 	auto text = lng_sure_delete_contact(
 		lt_contact,
